# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: designers.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='designers.proto',
  package='',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x0f\x64\x65signers.proto\"Q\n\x11\x44\x65signerInfoProto\x12\x10\n\x08\x64\x65signer\x18\x01 \x01(\t\x12\x0c\n\x04link\x18\x02 \x01(\t\x12\x1c\n\x06\x61vatar\x18\x03 \x01(\x0b\x32\x0c.AvatarProto\" \n\x0b\x41vatarProto\x12\x11\n\tfile_name\x18\x01 \x01(\t')
)




_DESIGNERINFOPROTO = _descriptor.Descriptor(
  name='DesignerInfoProto',
  full_name='DesignerInfoProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='designer', full_name='DesignerInfoProto.designer', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='link', full_name='DesignerInfoProto.link', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='avatar', full_name='DesignerInfoProto.avatar', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19,
  serialized_end=100,
)


_AVATARPROTO = _descriptor.Descriptor(
  name='AvatarProto',
  full_name='AvatarProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='AvatarProto.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=134,
)

_DESIGNERINFOPROTO.fields_by_name['avatar'].message_type = _AVATARPROTO
DESCRIPTOR.message_types_by_name['DesignerInfoProto'] = _DESIGNERINFOPROTO
DESCRIPTOR.message_types_by_name['AvatarProto'] = _AVATARPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DesignerInfoProto = _reflection.GeneratedProtocolMessageType('DesignerInfoProto', (_message.Message,), dict(
  DESCRIPTOR = _DESIGNERINFOPROTO,
  __module__ = 'designers_pb2'
  # @@protoc_insertion_point(class_scope:DesignerInfoProto)
  ))
_sym_db.RegisterMessage(DesignerInfoProto)

AvatarProto = _reflection.GeneratedProtocolMessageType('AvatarProto', (_message.Message,), dict(
  DESCRIPTOR = _AVATARPROTO,
  __module__ = 'designers_pb2'
  # @@protoc_insertion_point(class_scope:AvatarProto)
  ))
_sym_db.RegisterMessage(AvatarProto)


# @@protoc_insertion_point(module_scope)
