{% extends "_base.html" %}
{% block title %}Waterfall{% endblock %}

{% block content_name %}
<b>Waterfall</b>
{% endblock %}
{% block content %}
  {% for font_class in css_font_classes or css_font_classes_before or css_font_classes_after %}
    {% for pt_size in [7, 10, 11, 12, 14, 16, 18, 21, 27, 32] %}
      <div class="box">
	<div class="box-title">{{ font_class.selector }} {{ pt_size }}pt</div>
	<span class="{{ font_class.selector }}" style="font-size: {{ pt_size }}pt">
            QUICK WAFTING ZEPHYRS VEX BOLD JIM.<br>
            quick wafting zephyrs vex bold jim.<br>
            $14.95<br>
          </span>
       </div>
    {% endfor %}
  {% endfor %}
{% endblock %}
